/******************************************************************************
 * Name         : usetab.c
 * Title        : Useful USE lookup tables (copied from use.c)
 * Author       : James McCarthy
 * Created      : 28/04/2006
 *
 * Copyright    : 2006-2010 by Imagination Technologies Limited.
 *              : All rights reserved. No part of this software, either
 *              : material or conceptual may be copied or distributed,
 *              : transmitted, transcribed, stored in a retrieval system or
 *              : translated into any human or computer language in any form
 *              : by any means, electronic, mechanical, manual or otherwise,
 *              : or disclosed to third parties without the express written
 *              : permission of Imagination Technologies Limited, Home Park
 *              : Estate, Kings Langley, Hertfordshire, WD4 8LZ, U.K.
 *
 * Modifications:-
 * $Log: usetab.c $
 *****************************************************************************/

#include "sgxsupport.h"

#include "sgxdefs.h"

#include "use.h"
#include "usetab.h"
#include "useasm.h"

#if defined(USER) || defined(DEBUG)

/* Friendly versions of the register types. */
IMG_INTERNAL const IMG_PCHAR pszTypeToString[USEASM_REGTYPE_MAXIMUM] =
{
	"temporary",			/* USEASM_REGTYPE_TEMP */
	"output",				/* USEASM_REGTYPE_OUTPUT */
	"primary attribute",	/* USEASM_REGTYPE_PRIMATTR */
	"secondary attribute",	/* USEASM_REGTYPE_SECATTR */
	"index",				/* USEASM_REGTYPE_INDEX */
	"global",				/* USEASM_REGTYPE_GLOBAL */
	"constant",				/* USEASM_REGTYPE_FPCONSTANT */
	"internal",				/* USEASM_REGTYPE_FPINTERNAL */
	"immediate",			/* USEASM_REGTYPE_IMMEDIATE */
	"link",					/* USEASM_REGTYPE_LINK */
	"drc",					/* USEASM_REGTYPE_DRC */
	"label",				/* USEASM_REGTYPE_LABEL */
	"predicate",			/* USEASM_REGTYPE_PREDICATE */
	"clipplane",			/* USEASM_REGTYPE_CLIPPLANE */
	"address mode",			/* USEASM_REGTYPE_ADDRESSMODE */
	"swizzle",				/* USEASM_REGTYPE_SWIZZLE */
	"8-bit source select",	/* USEASM_REGTYPE_SOPSRCSEL */
	"filter coefficent",	/* USEASM_REGTYPE_FILTERCOEFF */
	"label",				/* USEASM_REGTYPE_LABEL_WITH_OFFSET */
	"named temporary",		/* USEASM_REGTYPE_TEMP_NAMED */
	"float immediate",		/* USEASM_REGTYPE_FLOATIMMEDIATE */
};
#endif /* defined(USER) || defined(DEBUG) */

#if defined(SUPPORT_SGX_FEATURE_USE_VEC34)
IMG_INTERNAL const USETAB_SPECIAL_CONST g_asVecHardwareConstants[SGXVEC_USE_SPECIAL_NUM_FLOAT_CONSTANTS] =
{
	/* SGXVEC_USE_SPECIAL_CONSTANT_ZERO_ZERO */
	{{0x00000000, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_ZERO_ONE */
	{{0x00000000, 0x3F800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_ONE_ZERO */
	{{0x3F800000, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_ONE_ONE */
	{{0x3F800000, 0x3F800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_TWO_FOUR */
	{{0x40000000, 0x40800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_8_16 */
	{{0x41000000, 0x41800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_32_64 */
	{{0x42000000, 0x42800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_128_256 */
	{{0x43000000, 0x43800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_512_1024 */
	{{0x44000000, 0x44800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_2048_4096 */
	{{0x45000000, 0x45800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_8192_16384 */
	{{0x46000000, 0x46800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_32768_65536 */
	{{0x47000000, 0x47800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER2_1OVER4 */
	{{0x3F000000, 0x3E800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER8_1OVER16 */
	{{0x3E000000, 0x3D800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER32_1OVER64 */
	{{0x3D000000, 0x3C800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER128_1OVER256 */
	{{0x3C000000, 0x3B800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER512_1OVER1024 */
	{{0x3B000000, 0x3A800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER2048_1OVER4096 */
	{{0x3A000000, 0x39800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER8192_1OVER16384 */
	{{0x39000000, 0x38800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER32768_1OVER65536 */
	{{0x38000000, 0x37800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_E_1OVERE */
	{{0x402DF854, 0x3EBC5AB2} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_SQRT2_1OVERSQRT2 */
	{{0x3FB504F3, 0x3F3504F3} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_PI_PIOVER2 */
	{{0x40490FDB, 0x3FC90FDB} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_PIOVER4_PIOVER8 */
	{{0x3F490FDB, 0x3EC90FDB} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_2PI_4PI */
	{{0x40C90FDB, 0x41490FDB} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_8PI_ZERO	 */
	{{0x41C90FDB, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER65536_1OVER32768 */
	{{0x37800000, 0x38000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_1OVER65535_1OVER32767 */
	{{0x37800080, 0x38000100} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_TAYLORSERIES1 */
	{{0x35D00D01, 0x37B60B61} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_TAYLORSERIES2 */
	{{0x39888889, 0x3B2AAAAB} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_TAYLORSERIES3 */
	{{0x3CAAAAAB, 0x3E000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_TAYLORSERIES4 */
	{{0x3F000000, 0x3F800000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_ZERO_ZERO_ZERO_ONE */
	{{0x00000000, 0x3C000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_ZERO_ZERO_ZERO_ZERO */
	{{0x00000000, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_ONE_ONE_ONE_ONE */
	{{0x3C003C00, 0x3C003C00} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_2_4_8_16	 */
	{{0x44004000, 0x4C004800} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_32_64_128_256 */
	{{0x54005000, 0x5C005800} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_512_1024_2048_4096 */
	{{0x64006000, 0x6C006800} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_8192_16384_32768_ZERO */
	{{0x74007000, 0x00007800} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_1OVER2_1OVER4_1OVER8_1OVER16 */
	{{0x34003800, 0x2C003000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_1OVER32_1OVER64_1OVER128_1OVER256 */
	{{0x24002800, 0x1C002000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_1OVER512_1OVER1024_1OVER2048_1OVER4096 */
	{{0x14001800, 0x0C001000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_1OVER8192_1OVER16384_ZERO_ZERO */
	{{0x04000800, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_E_1OVERE_ZERO_ZERO */
	{{0x35E2416F, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_SQRT2_1OVERSQRT2_ZERO_ZERO */
	{{0x39A83DA8, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_PI_PIOVER2_PIOVER4_PIOVER8 */
	{{0x3E484248, 0x36483A48} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_2PI_4PI_8PI_ZERO */
	{{0x4A484648, 0x00004E48} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_ZERO_ZERO_ZERO_ZERO_VER1 */
	{{0x00000000, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_TAYLORSERIES12 */
	{{0x00000000, 0x19550C44} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_F16_TAYLORSERIES34 */
	{{0x30002555, 0x3C003800} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_RESERVED0 */
	{{0x00000000, 0x00000000} /* auValue */, IMG_TRUE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_RESERVED1 */
	{{0x00000000, 0x00000000} /* auValue */, IMG_TRUE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_RESERVED2 */
	{{0x00000000, 0x00000000} /* auValue */, IMG_TRUE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_RESERVED3 */
	{{0x00000000, 0x00000000} /* auValue */, IMG_TRUE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_RESERVED4 */
	{{0x00000000, 0x00000000} /* auValue */, IMG_TRUE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_RESERVED5 */
	{{0x00000000, 0x00000000} /* auValue */, IMG_TRUE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_FFFFFFFF */
	{{0xFFFFFFFF, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_FFFFFFFF_VER1 */
	{{0xFFFFFFFF, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_FFFFFFFF_VER2 */
	{{0xFFFFFFFF, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_FFFFFFFF_VER3 */
	{{0xFFFFFFFF, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_7FFF7FFF */
	{{0x7FFF7FFF, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_7FFF7FFF_VER1 */
	{{0x7FFF7FFF, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_7FFF7FFF_VER2 */
	{{0x7FFF7FFF, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
	/* SGXVEC_USE_SPECIAL_CONSTANT_7FFF7FFF_VER3 */
	{{0x7FFF7FFF, 0x00000000} /* auValue */, IMG_FALSE /* bReserved */},
};

static const DUALISSUEVECTOR_SRCSLOT g_aeU__[] = {DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_UNDEF, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae0__[] = {DUALISSUEVECTOR_SRCSLOT_GPI0, DUALISSUEVECTOR_SRCSLOT_UNDEF, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae1__[] = {DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_UNDEF, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae2__[] = {DUALISSUEVECTOR_SRCSLOT_GPI2, DUALISSUEVECTOR_SRCSLOT_UNDEF, DUALISSUEVECTOR_SRCSLOT_UNDEF};

static const DUALISSUEVECTOR_SRCSLOT g_aeU1_[] = {DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae0U_[] = {DUALISSUEVECTOR_SRCSLOT_GPI0, DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae01_[] = {DUALISSUEVECTOR_SRCSLOT_GPI0, DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae02_[] = {DUALISSUEVECTOR_SRCSLOT_GPI0, DUALISSUEVECTOR_SRCSLOT_GPI2, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae1U_[] = {DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae12_[] = {DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_GPI2, DUALISSUEVECTOR_SRCSLOT_UNDEF};
static const DUALISSUEVECTOR_SRCSLOT g_ae2U_[] = {DUALISSUEVECTOR_SRCSLOT_GPI2, DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_UNDEF};

static const DUALISSUEVECTOR_SRCSLOT g_aeU12[] = {DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_GPI2};
static const DUALISSUEVECTOR_SRCSLOT g_ae0U2[] = {DUALISSUEVECTOR_SRCSLOT_GPI0, DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_GPI2};
static const DUALISSUEVECTOR_SRCSLOT g_ae01U[] = {DUALISSUEVECTOR_SRCSLOT_GPI0, DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE};
static const DUALISSUEVECTOR_SRCSLOT g_ae012[] = {DUALISSUEVECTOR_SRCSLOT_GPI0, DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_GPI2};
static const DUALISSUEVECTOR_SRCSLOT g_ae0U1[] = {DUALISSUEVECTOR_SRCSLOT_GPI0, DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_GPI1};
static const DUALISSUEVECTOR_SRCSLOT g_ae1U2[] = {DUALISSUEVECTOR_SRCSLOT_GPI1, DUALISSUEVECTOR_SRCSLOT_UNIFIEDSTORE, DUALISSUEVECTOR_SRCSLOT_GPI2};

IMG_INTERNAL const PCDUALISSUEVECTOR_SRCSLOT g_aapeDualIssueVectorPrimaryMap[SGXVEC_USE_DVEC_OP1_MAXIMUM_SRC_COUNT]
                                                                            [SGXVEC_USE0_DVEC_SRCCFG_COUNT] = 
{
	/* uOp1SrcCount == 1 */
	{
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_aeU__,
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_ae0__,
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_ae1__,
		/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	g_ae2__,
	},
	/* uOp1SrcCount == 2 */
	{
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_aeU1_,
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_ae0U_,
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_ae01_,
		/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	IMG_NULL,
	},
	/* uOp1SrcCount == 3 */
	{
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_aeU12,
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_ae0U2,
		/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_ae01U,
		/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	g_ae012,
	},
};

IMG_INTERNAL const PCDUALISSUEVECTOR_SRCSLOT g_aaapeDualIssueVectorSecondaryMap[SGXVEC_USE_DVEC_OP1_MAXIMUM_SRC_COUNT]
                                                                               [SGXVEC_USE_DVEC_OP2_MAXIMUM_SRC_COUNT]
																			   [SGXVEC_USE0_DVEC_SRCCFG_COUNT] = 
{
	/* uOp1SrcCount == 1 */
	{
		/* uOp2SrcCount == 1 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_ae0__,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_aeU__,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_aeU__,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	g_aeU__,
		},
		/* uOp2SrcCount == 2 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_ae01_,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_aeU1_,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_ae0U_,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	g_ae0U_,
		},
		/* uOp2SrcCount == 3 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_ae012,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_aeU12,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_ae0U2,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	g_ae0U1,
		},
	},
	/* uOp1SrcCount == 2 */
	{
		/* uOp2SrcCount == 1 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_ae0__,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_ae1__,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_aeU__,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	IMG_NULL,
		},
		/* uOp2SrcCount == 2 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_ae02_,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_ae12_,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_ae2U_,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	IMG_NULL,
		},
		/* uOp2SrcCount == 3 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	IMG_NULL,
		},
	},
	/* uOp1SrcCount == 3 */
	{
		/* uOp2SrcCount == 1 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	g_ae0__,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	g_ae1__,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	g_ae2__,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	g_aeU__,
		},
		/* uOp2SrcCount == 2 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	IMG_NULL,
		},
		/* uOp2SrcCount == 3 */
		{
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC0 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC1 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_SRC2 */	IMG_NULL,
			/* SGXVEC_USE0_DVEC_SRCCFG_NONE */	IMG_NULL,
		},
	},
};
#endif /* defined(SUPPORT_SGX_FEATURE_USE_VEC34) */

/* Floating point constants available directly in the hardware. */
IMG_INTERNAL const IMG_FLOAT g_pfHardwareConstants[USEASM_NUM_HW_SPECIAL_CONSTS] =
{
	0.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_ZERO1 */
	0.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_ZERO2 */
	0.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_ZERO3 */
	1.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1_1 */
	2.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT2 */
	4.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT4 */
	8.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT8 */
	16.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT16 */
	32.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT32 */
	64.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT64 */
	128.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT128 */
	256.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT256 */
	512.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT512 */
	1024.0f,				/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1024 */
	0.5f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER2 */
	0.25f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER4 */
	0.125f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER8 */
	0.0625f,				/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER16 */
	0.03125f,				/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER32 */
	0.015625f,				/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER64 */
	0.0078125f,				/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER128 */
	0.00390625f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER256 */
	0.001953125f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER512 */
	0.0009765625f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER1024 */
	0.00048828125f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER2048 */
	0.000244140625f,		/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER4096 */
	0.0001220703125f,		/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER8192 */
	0.00006103515625f,		/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER16384 */
	2.718281828f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOATE */
	0.367879441f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVERE */
	1.414213562f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOATSQR2 */
	0.707106781f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVERSQR2 */
	0.785398163f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOATPIOVER4 */
	1.570796327f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOATPIOVER2 */
	3.141592654f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOATPI */
	0.318309886f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVERPI */
	0.636619772f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT2OVERPI */
	1.273239545f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT4OVERPI */
	6.283185307f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT2TIMESPI */
	0.00001525878907f,		/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1OVER65536 */
	0.000001550099206f,		/* EURASIA_USE_SPECIAL_CONSTANT_FLOATTAYLOR2X */
	0.00002170138889f,		/* EURASIA_USE_SPECIAL_CONSTANT_FLOATTAYLOR2Y */
	0.0002604166667f,		/* EURASIA_USE_SPECIAL_CONSTANT_FLOATTAYLOR2W */
	0.002604166667f,		/* EURASIA_USE_SPECIAL_CONSTANT_FLOATTAYLOR2Z */
	0.0208333333f,			/* EURASIA_USE_SPECIAL_CONSTANT_FLOATTAYLOR3X */
	0.125f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOATTAYLOR3Y */
	0.5f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOATTAYLOR3W */
	1.0f,					/* EURASIA_USE_SPECIAL_CONSTANT_FLOATTAYLOR3Z */
	0,						/* EURASIA_USE_SPECIAL_CONSTANT_ZERO */
	0,						/* EURASIA_USE_SPECIAL_CONSTANT_ZERO4 */
	0,						/* EURASIA_USE_SPECIAL_CONSTANT_ZERO5 */
	0,						/* EURASIA_USE_SPECIAL_CONSTANT_ZERO6 */
	1,						/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1 */
	1,						/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1_2 */
	1,						/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1_3 */
	1,						/* EURASIA_USE_SPECIAL_CONSTANT_FLOAT1_4 */
};

#if defined(USER)
#define DCL_OPCODE_NAME(NAME)	NAME,
#else /* defined(USER) */
#define DCL_OPCODE_NAME(NAME)
#endif /* defined(USER) */

/* Instruction Description Table */
IMG_INTERNAL const USE_INSTDESC g_asUseInstDesc[] =
{
	/* USEASM_OP_FMAD */
	{
		/* pszName */
		DCL_OPCODE_NAME("fmad")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FADM */
	{
		/* pszName */
		DCL_OPCODE_NAME("fadm")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FMSA */
	{
		/* pszName */
		DCL_OPCODE_NAME("fmsa")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FFRC */
	{
		/* pszName */
		DCL_OPCODE_NAME("ffrc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FRCP */
	{
		/* pszName */
		DCL_OPCODE_NAME("frcp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORC10 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK_FOR_VECTOR |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FRSQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("frsq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORC10 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK_FOR_VECTOR |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FLOG */
	{
		/* pszName */
		DCL_OPCODE_NAME("flog")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORC10 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK_FOR_VECTOR |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FEXP */
	{
		/* pszName */
		DCL_OPCODE_NAME("fexp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORC10 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK_FOR_VECTOR |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FSQRT */
	{
		/* pszName */
		DCL_OPCODE_NAME("fsqrt")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORC10 |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FSIN */
	{
		/* pszName */
		DCL_OPCODE_NAME("fsin")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORC10 |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FCOS */
	{
		/* pszName */
		DCL_OPCODE_NAME("fcos")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORC10 |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FDP */
	{
		/* pszName */
		DCL_OPCODE_NAME("fdp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FMIN */
	{
		/* pszName */
		DCL_OPCODE_NAME("fmin")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FMAX */
	{
		/* pszName */
		DCL_OPCODE_NAME("fmax")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FDSX */
	{
		/* pszName */
		DCL_OPCODE_NAME("fdsx")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FDSY */
	{
		/* pszName */
		DCL_OPCODE_NAME("fdsy")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FMAD16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fmad16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FMINMAX */
	{
		/* pszName */
		DCL_OPCODE_NAME("fminmax")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FMAXMIN */
	{
		/* pszName */
		DCL_OPCODE_NAME("fmaxmin")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_EFO */
	{
		/* pszName */
		DCL_OPCODE_NAME("efo")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_PCKF16F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pckf16f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKF16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pckf16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKF16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pckf16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKF16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pckf16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKU16F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcku16f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKU16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcku16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKU16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcku16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKU16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcku16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKS16F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcks16f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKS16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcks16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKS16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcks16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKS16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcks16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKU8F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcku8f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKU8F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcku8f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKU8U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcku8u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKU8S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcku8s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKS8F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcks8f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKS8F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcks8f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKS8U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcks8u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKS8S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcks8s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKO8F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcko8f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKO8F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcko8f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKO8U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcko8u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKO8S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcko8s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKC10F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pckc10f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKC10F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pckc10f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKC10U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pckc10u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_PCKC10S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("pckc10s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF32F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf32f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF32F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf32f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF32U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf32u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF32S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf32s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF32U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf32u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF32S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf32s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF32O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf32o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF32C10 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf32c10")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF16U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf16u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF16S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf16s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF16O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf16o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKF16C10 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckf16c10")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU16U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku16u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU16S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku16s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU16O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku16o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU16C10 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku16c10")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKS16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcks16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKS16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcks16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKS16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcks16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKS16U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcks16u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKS16S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcks16s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKS16O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcks16o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKS16C10 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcks16c10")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU8U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku8u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKC10C10 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpckc10c10")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_AND */
	{
		/* pszName */
		DCL_OPCODE_NAME("and")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_SHOW_SRC2_IMMEDIATES_IN_HEX |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_OR */
	{
		/* pszName */
		DCL_OPCODE_NAME("or")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_SHOW_SRC2_IMMEDIATES_IN_HEX |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_XOR */
	{
		/* pszName */
		DCL_OPCODE_NAME("xor")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_SHOW_SRC2_IMMEDIATES_IN_HEX |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SHL */
	{
		/* pszName */
		DCL_OPCODE_NAME("shl")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_SHOW_SRC2_IMMEDIATES_IN_HEX |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SHR */
	{
		/* pszName */
		DCL_OPCODE_NAME("shr")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_SHOW_SRC2_IMMEDIATES_IN_HEX |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ASR */
	{
		/* pszName */
		DCL_OPCODE_NAME("asr")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_SHOW_SRC2_IMMEDIATES_IN_HEX |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ROL */
	{
		/* pszName */
		DCL_OPCODE_NAME("rol")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_SHOW_SRC2_IMMEDIATES_IN_HEX |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_RLP */
	{
		/* pszName */
		DCL_OPCODE_NAME("rlp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SOP2 */
	{
		/* pszName */
		DCL_OPCODE_NAME("sop2")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_SOP2WM */
	{
		/* pszName */
		DCL_OPCODE_NAME("sop2wm")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_SOP3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("sop3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		8,
	},
	/* USEASM_OP_U8DOT3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("u8dot3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_U8DOT4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("u8dot4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_U16DOT3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("u16dot3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_U16DOT4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("u16dot4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_U8DOT3OFF */
	{
		/* pszName */
		DCL_OPCODE_NAME("u8dot3off")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_U8DOT4OFF */
	{
		/* pszName */
		DCL_OPCODE_NAME("u8dot4off")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_U16DOT3OFF */
	{
		/* pszName */
		DCL_OPCODE_NAME("u16dot3off")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_U16DOT4OFF */
	{
		/* pszName */
		DCL_OPCODE_NAME("u16dot4off")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_IMA8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("ima8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		10,
	},
	/* USEASM_OP_IMA16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("ima16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_IMAE */
	{
		/* pszName */
		DCL_OPCODE_NAME("imae")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		8,
	},
	/* USEASM_OP_IMA32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("ima32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_IDIV */
	{
		/* pszName */
		DCL_OPCODE_NAME("idiv")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_ADIF */
	{
		/* pszName */
		DCL_OPCODE_NAME("adif")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ADIFSUM */
	{
		/* pszName */
		DCL_OPCODE_NAME("adifsum")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FIRV */
	{
		/* pszName */
		DCL_OPCODE_NAME("firv")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		8,
	},
	/* USEASM_OP_FIRVH */
	{
		/* pszName */
		DCL_OPCODE_NAME("firvh")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_BILIN */
	{
		/* pszName */
		DCL_OPCODE_NAME("bilin")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		9,
	},
	/* USEASM_OP_FIRH */
	{
		/* pszName */
		DCL_OPCODE_NAME("firh")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		9,
	},
	/* USEASM_OP_FIRHH */
	{
		/* pszName */
		DCL_OPCODE_NAME("firhh")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		9,
	},
	/* USEASM_OP_SSUM16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("ssum16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		9,
	},
	/* USEASM_OP_SMP1D */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp1d")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_SMP2D */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp2d")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_SMP3D */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp3d")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_SMP1DBIAS */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp1dbias")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_SMP2DBIAS */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp2dbias")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_SMP3DBIAS */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp3dbias")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_SMP1DREPLACE */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp1dreplace")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_SMP2DREPLACE */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp2dreplace")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_SMP3DREPLACE */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp3dreplace")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_SMP1DGRAD */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp1dgrad")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_SMP2DGRAD */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp2dgrad")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_SMP3DGRAD */
	{
		/* pszName */
		DCL_OPCODE_NAME("smp3dgrad")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USC_DESCFLAG_TEXTURESAMPLE |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_LDAB */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldab")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDAW */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldaw")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDAD */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldad")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDAQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldaq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_STAB */
	{
		/* pszName */
		DCL_OPCODE_NAME("stab")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STAW */
	{
		/* pszName */
		DCL_OPCODE_NAME("staw")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STAD */
	{
		/* pszName */
		DCL_OPCODE_NAME("stad")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STAQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("staq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_LDLB */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldlb")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDLW */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldlw")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDLD */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldld")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDLQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldlq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_STLB */
	{
		/* pszName */
		DCL_OPCODE_NAME("stlb")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STLW */
	{
		/* pszName */
		DCL_OPCODE_NAME("stlw")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STLD */
	{
		/* pszName */
		DCL_OPCODE_NAME("stld")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STLQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("stlq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_LDTB */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldtb")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDTW */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldtw")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDTD */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldtd")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_LDTQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldtq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_STTB */
	{
		/* pszName */
		DCL_OPCODE_NAME("sttb")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STTW */
	{
		/* pszName */
		DCL_OPCODE_NAME("sttw")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STTD */
	{
		/* pszName */
		DCL_OPCODE_NAME("sttd")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STTQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("sttq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ELDD */
	{
		/* pszName */
		DCL_OPCODE_NAME("eldd")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_ELDQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("eldq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_BA */
	{
		/* pszName */
		DCL_OPCODE_NAME("ba")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		1,
	},
	/* USEASM_OP_BR */
	{
		/* pszName */
		DCL_OPCODE_NAME("br")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		1,
	},
	/* USEASM_OP_LAPC */
	{
		/* pszName */
		DCL_OPCODE_NAME("lapc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		0,
	},
	/* USEASM_OP_BEXCEPTION */
	{
		/* pszName */
		DCL_OPCODE_NAME("bexception")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		0,
	},
	/* USEASM_OP_IDF */
	{
		/* pszName */
		DCL_OPCODE_NAME("idf")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_WDF */
	{
		/* pszName */
		DCL_OPCODE_NAME("wdf")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		1,
	},
	/* USEASM_OP_LIMM */
	{
		/* pszName */
		DCL_OPCODE_NAME("limm")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_EMITPIXEL1 */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitpix1")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_EMITPIXEL2 */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitpix2")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_EMITPIXEL */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitpix")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_EMITSTATE */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitst")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_EMITVERTEX */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitvtx")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_EMITPRIMITIVE */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitprimitive")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_EMITPDS */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitpds")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_MOV */
	{
		/* pszName */
		DCL_OPCODE_NAME("mov")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_SHOW_SRC2_IMMEDIATES_IN_HEX |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FMOV */
	{
		/* pszName */
		DCL_OPCODE_NAME("fmov")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FMUL */
	{
		/* pszName */
		DCL_OPCODE_NAME("fmul")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FADD */
	{
		/* pszName */
		DCL_OPCODE_NAME("fadd")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FSSQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("fssq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_LABEL */
	{
		/* pszName */
		DCL_OPCODE_NAME("label")
		/* uDescFlags */
		(0),
		/* uArgCount */
		1,
	},
	/* USEASM_OP_FSUB */
	{
		/* pszName */
		DCL_OPCODE_NAME("fsub")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SETPGT */
	{
		/* pszName */
		DCL_OPCODE_NAME("setpgt")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SETPGTE */
	{
		/* pszName */
		DCL_OPCODE_NAME("setpgte")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SETPEQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("setpeq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SETPLT */
	{
		/* pszName */
		DCL_OPCODE_NAME("setplt")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SETPLTE */
	{
		/* pszName */
		DCL_OPCODE_NAME("setplte")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SETPNEQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("setpneq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SDM */
	{
		/* pszName */
		DCL_OPCODE_NAME("sdm")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_SDA */
	{
		/* pszName */
		DCL_OPCODE_NAME("sda")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_AMM */
	{
		/* pszName */
		DCL_OPCODE_NAME("amm")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_SMM */
	{
		/* pszName */
		DCL_OPCODE_NAME("smm")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_AMS */
	{
		/* pszName */
		DCL_OPCODE_NAME("ams")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_SMS */
	{
		/* pszName */
		DCL_OPCODE_NAME("sms")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_DMA */
	{
		/* pszName */
		DCL_OPCODE_NAME("dma")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_NOP */
	{
		/* pszName */
		DCL_OPCODE_NAME("nop")
		/* uDescFlags */
		(0),
		/* uArgCount */
		0,
	},
	/* USEASM_OP_FTZ */
	{
		/* pszName */
		DCL_OPCODE_NAME("ftz")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FTNZ */
	{
		/* pszName */
		DCL_OPCODE_NAME("ftnz")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_U16TZ */
	{
		/* pszName */
		DCL_OPCODE_NAME("u16tz")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_U16TNZ */
	{
		/* pszName */
		DCL_OPCODE_NAME("u16tnz")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_SMOA */
	{
		/* pszName */
		DCL_OPCODE_NAME("smoa")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		8,
	},
	/* USEASM_OP_SMR */
	{
		/* pszName */
		DCL_OPCODE_NAME("smr")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_SMLSI */
	{
		/* pszName */
		DCL_OPCODE_NAME("smlsi")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		11,
	},
	/* USEASM_OP_SMBO */
	{
		/* pszName */
		DCL_OPCODE_NAME("smbo")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_IMO */
	{
		/* pszName */
		DCL_OPCODE_NAME("imo")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_SETFC */
	{
		/* pszName */
		DCL_OPCODE_NAME("setfc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_FDP3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fdp3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FDP4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fdp4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FDPC */
	{
		/* pszName */
		DCL_OPCODE_NAME("fdpc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FDPC3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fdpc3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FDPC4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fdpc4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FDDP */
	{
		/* pszName */
		DCL_OPCODE_NAME("fddp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_FDDP3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fddp3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_FDDP4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fddp4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_FDDPC */
	{
		/* pszName */
		DCL_OPCODE_NAME("fddpc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_FDDPC3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fddpc3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_FDDPC4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fddpc4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_LOCK */
	{
		/* pszName */
		DCL_OPCODE_NAME("lock")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		1,
	},
	/* USEASM_OP_RELEASE */
	{
		/* pszName */
		DCL_OPCODE_NAME("release")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		1,
	},
	/* USEASM_OP_FSUBFLR */
	{
		/* pszName */
		DCL_OPCODE_NAME("fsubflr")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK_FOR_VECTOR |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_LDR */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldr")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_STR */
	{
		/* pszName */
		DCL_OPCODE_NAME("str")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_PCOEFF */
	{
		/* pszName */
		DCL_OPCODE_NAME("pcoeff")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_PTOFF */
	{
		/* pszName */
		DCL_OPCODE_NAME("ptoff")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		0,
	},
	/* USEASM_OP_ATST8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("atst8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_DEPTHF */
	{
		/* pszName */
		DCL_OPCODE_NAME("depthf")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_WOP */
	{
		/* pszName */
		DCL_OPCODE_NAME("wop")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		1,
	},
	/* USEASM_OP_SETM */
	{
		/* pszName */
		DCL_OPCODE_NAME("setm")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		1,
	},
	/* USEASM_OP_PHAS */
	{
		/* pszName */
		DCL_OPCODE_NAME("phas")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_PHASIMM */
	{
		/* pszName */
		DCL_OPCODE_NAME("phas")
		/* uDescFlags */
		(0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_AINTRP1 */
	{
		/* pszName */
		DCL_OPCODE_NAME("aintrp1")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_AINTRP2 */
	{
		/* pszName */
		DCL_OPCODE_NAME("aintrp2")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_AADD */
	{
		/* pszName */
		DCL_OPCODE_NAME("aadd")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_ASUB */
	{
		/* pszName */
		DCL_OPCODE_NAME("asub")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_ASOP */
	{
		/* pszName */
		DCL_OPCODE_NAME("asop")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ARSOP */
	{
		/* pszName */
		DCL_OPCODE_NAME("arsop")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_ALRP */
	{
		/* pszName */
		DCL_OPCODE_NAME("alrp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_LRP1 */
	{
		/* pszName */
		DCL_OPCODE_NAME("lrp1")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_LRP2 */
	{
		/* pszName */
		DCL_OPCODE_NAME("lrp2")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		8,
	},
	/* USEASM_OP_ASOP2 */
	{
		/* pszName */
		DCL_OPCODE_NAME("asop2")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_IADD16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("iadd16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ISUB16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("isub16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IMUL16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("imul16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IADDU16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("iaddu16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ISUBU16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("isubu16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IMULU16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("imulu16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IADD32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("iadd32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IADD8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("iadd8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ISUB8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("isub8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IMUL8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("imul8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IMULU8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("imulu8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IMOV16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("imov16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IADDU32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("iaddu32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_PADDING */
	{
		/* pszName */
		DCL_OPCODE_NAME("nop")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		0,
	},
	/* USEASM_OP_MOVC */
	{
		/* pszName */
		DCL_OPCODE_NAME("movc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_SIMPLE_TESTS |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_MOVMSK */
	{
		/* pszName */
		DCL_OPCODE_NAME("movmsk")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_FPMA */
	{
		/* pszName */
		DCL_OPCODE_NAME("fpma")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		10,
	},
	/* USEASM_OP_IADDU8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("iaddu8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ISUBU8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("isubu8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FPMUL8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fpmul8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FPADD8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fpadd8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FPSUB8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fpsub8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_SABLND */
	{
		/* pszName */
		DCL_OPCODE_NAME("sablnd")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_FNRM32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fnrm32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_FNRM16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("fnrm16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMPFORF16 |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_MOEST */
	{
		/* pszName */
		DCL_OPCODE_NAME("moest")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_CF */
	{
		/* pszName */
		DCL_OPCODE_NAME("cf")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_CI */
	{
		/* pszName */
		DCL_OPCODE_NAME("ci")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_CFI */
	{
		/* pszName */
		DCL_OPCODE_NAME("cfi")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_EMITMTEVERTEX */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitmtevtx")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_EMITMTESTATE */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitmtest")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_EMITVCBVERTEX */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitvcbvtx")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_EMITVCBSTATE */
	{
		/* pszName */
		DCL_OPCODE_NAME("emitvcbst")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_VDP3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vdp3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 USE_DESCFLAG_DISASM_THREE_COMPONENT_SWIZZLE |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_VDP4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vdp4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_VMAD3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmad3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 USE_DESCFLAG_DISASM_THREE_COMPONENT_SWIZZLE |
		 0),
		/* uArgCount */
		8,
	},
	/* USEASM_OP_VMAD4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmad4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		8,
	},
	/* USEASM_OP_VMUL3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmul3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 USE_DESCFLAG_DISASM_THREE_COMPONENT_SWIZZLE |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VMUL4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmul4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VADD3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vadd3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_THREE_COMPONENT_SWIZZLE |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VADD4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vadd4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VSUB3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vsub3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_THREE_COMPONENT_SWIZZLE |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VSUB4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vsub4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VSSQ3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vssq3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_THREE_COMPONENT_SWIZZLE |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VSSQ4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vssq4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VMOV3 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmov3")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_THREE_COMPONENT_SWIZZLE |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VMOV4 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmov4")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VMOV */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmov")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VMOVC */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmovc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_SIMPLE_TESTS |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VMOVCU8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmovcu8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_SIMPLE_TESTS |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VMAD */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmad")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_VMUL */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmul")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VADD */
	{
		/* pszName */
		DCL_OPCODE_NAME("vadd")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VFRC */
	{
		/* pszName */
		DCL_OPCODE_NAME("vfrc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VMIN */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmin")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VMAX */
	{
		/* pszName */
		DCL_OPCODE_NAME("vmax")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VDSX */
	{
		/* pszName */
		DCL_OPCODE_NAME("vdsx")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VDSY */
	{
		/* pszName */
		DCL_OPCODE_NAME("vdsy")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VDP */
	{
		/* pszName */
		DCL_OPCODE_NAME("vdp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VSUB */
	{
		/* pszName */
		DCL_OPCODE_NAME("vsub")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16MAD */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16mad")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		7,
	},
	/* USEASM_OP_VF16MUL */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16mul")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16ADD */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16add")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16FRC */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16frc")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16DSX */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16dsx")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16DSY */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16dsy")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16MIN */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16min")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16MAX */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16max")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16DP */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16dp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VF16SUB */
	{
		/* pszName */
		DCL_OPCODE_NAME("vf16sub")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_VRCP */
	{
		/* pszName */
		DCL_OPCODE_NAME("vrcp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_VRSQ */
	{
		/* pszName */
		DCL_OPCODE_NAME("vrsq")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_VLOG */
	{
		/* pszName */
		DCL_OPCODE_NAME("vlog")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_VEXP */
	{
		/* pszName */
		DCL_OPCODE_NAME("vexp")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		2,
	},
	/* USEASM_OP_VPCKU8U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku8u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU8S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku8s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU8O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku8o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU8U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku8u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU8S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku8s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU8F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku8f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU8F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku8f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_VPCKS8U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks8u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS8S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks8s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS8O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks8o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS8U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks8u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS8S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks8s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS8F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks8f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS8F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks8f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_VPCKO8U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcko8u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKO8S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcko8s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKO8O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcko8o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKO8U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcko8u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKO8S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcko8s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKO8F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcko8f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKO8F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcko8f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_VPCKU16U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku16u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU16S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku16s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU16O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku16o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKU16F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcku16f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_VPCKS16U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks16u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS16S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks16s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS16O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks16o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKS16F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpcks16f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_VPCKF16U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf16u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF16S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf16s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF16O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf16o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF16U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf16u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF16S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf16s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF16F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf16f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF16F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf16f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_VPCKF16C10 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf16c10")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF32U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf32u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF32S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf32s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF32O8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf32o8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF32U16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf32u16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF3S16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf32s16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF32F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf32f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKF32F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf32f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_VPCKF32C10 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckf32c10")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKC10F16 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckc10f16")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_VPCKC10F32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckc10f32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_VPCKC10C10 */
	{
		/* pszName */
		DCL_OPCODE_NAME("vpckc10c10")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_MASK |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_IDXSCR */
	{
		/* pszName */
		DCL_OPCODE_NAME("idxscr")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_IDXSCW */
	{
		/* pszName */
		DCL_OPCODE_NAME("idxscw")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_SPRVV */
	{
		/* pszName */
		DCL_OPCODE_NAME("sprvv")
		/* uDescFlags */
		(0),
		/* uArgCount */
		0,
	},
	/* USEASM_OP_ISUB32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("isub32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_ISUBU32 */
	{
		/* pszName */
		DCL_OPCODE_NAME("isubu32")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_CNDST */
	{
		/* pszName */
		DCL_OPCODE_NAME("cndst")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_CNDEF */
	{
		/* pszName */
		DCL_OPCODE_NAME("cndef")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_CNDSM */
	{
		/* pszName */
		DCL_OPCODE_NAME("cndsm")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_CNDLT */
	{
		/* pszName */
		DCL_OPCODE_NAME("cndlt")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		5,
	},
	/* USEASM_OP_CNDEND */
	{
		/* pszName */
		DCL_OPCODE_NAME("cndend")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		3,
	},
	/* USEASM_OP_LDATOMIC */
	{
		/* pszName */
		DCL_OPCODE_NAME("ldatomic")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 0),
		/* uArgCount */
		6,
	},
	/* USEASM_OP_STATOMIC */
	{
		/* pszName */
		DCL_OPCODE_NAME("statomic")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_DISASM_MEMORY_ST |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKU8S8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcku8s8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	},
	/* USEASM_OP_UNPCKS8U8 */
	{
		/* pszName */
		DCL_OPCODE_NAME("unpcks8u8")
		/* uDescFlags */
		(USE_DESCFLAG_NOSCHED |
		 USE_DESCFLAG_NOSCHEDPLUS |
		 USE_DESCFLAG_SKIPINV |
		 USE_DESCFLAG_REPEAT |
		 USE_DESCFLAG_DISASM_ALWAYS_SHOW_SRCCOMP |
		 0),
		/* uArgCount */
		4,
	}
};

/* Replacement functions */
/*****************************************************************************
 FUNCTION	: OpcodeArgumentCount

 PURPOSE	: Get number of operands suppported by opcode

 PARAMETERS	: uOpcode    - Opcode

 RETURNS	: Maximum number of operands
*****************************************************************************/
IMG_INTERNAL
IMG_UINT32 OpcodeArgumentCount(IMG_UINT32 uOpcode)
{
	return g_asUseInstDesc[uOpcode].uArgCount;
}

#if defined(USER)
/*****************************************************************************
 FUNCTION	: OpcodeName

 PURPOSE	: Get string reprentation of opcode

 PARAMETERS	: uOpcode    - Opcode

 RETURNS	: String name of opcode
*****************************************************************************/
IMG_INTERNAL
IMG_PCHAR OpcodeName(IMG_UINT32 uOpcode)
{
	return g_asUseInstDesc[uOpcode].pszName;
}
#endif /* defined(USER) */

/*****************************************************************************
 FUNCTION	: OpcodeAcceptsSkipInv

 PURPOSE	: Test whether an instruction supports skip-invalid flags

 PARAMETERS	: uOpcode    - Opcode to test

 RETURNS	: IMG_TRUE iff instruction supports the flag
*****************************************************************************/
IMG_INTERNAL
IMG_BOOL OpcodeAcceptsSkipInv(IMG_UINT32 uOpcode)
{
	if (g_asUseInstDesc[uOpcode].uDescFlags & USE_DESCFLAG_SKIPINV)
		return IMG_TRUE;
	else
		return IMG_FALSE;
}

/*****************************************************************************
 FUNCTION	: OpcodeAcceptsNoSched

 PURPOSE	: Test whether an instruction supports no-sched flags

 PARAMETERS	: uOpcode    - Opcode to test

 RETURNS	: IMG_TRUE iff instruction supports the flag
*****************************************************************************/
IMG_INTERNAL
IMG_BOOL OpcodeAcceptsNoSched(IMG_UINT32 uOpcode)
{
	if (g_asUseInstDesc[uOpcode].uDescFlags & USE_DESCFLAG_NOSCHED)
		return IMG_TRUE;
	else
		return IMG_FALSE;
}

/*****************************************************************************
 FUNCTION	: OpcodeAcceptsNoSchedEnhanced

 PURPOSE	: Test whether an instruction supports enhanced no-sched flags

 PARAMETERS	: uOpcode    - Opcode to test

 RETURNS	: IMG_TRUE iff instruction supports the flag
*****************************************************************************/
IMG_INTERNAL
IMG_BOOL OpcodeAcceptsNoSchedEnhanced(IMG_UINT32 uOpcode)
{
	if (g_asUseInstDesc[uOpcode].uDescFlags & USE_DESCFLAG_NOSCHEDPLUS)
		return IMG_TRUE;
	else
		return IMG_FALSE;
}

/*****************************************************************************
 FUNCTION	: OpcodeDescFlags

 PURPOSE	: Returns the mask of opcode description flags for a specific
			  opcode.

 PARAMETERS	: uOpcode    - Opcode to test

 RETURNS	: The mask of flags.
*****************************************************************************/
IMG_INTERNAL
IMG_UINT32 OpcodeDescFlags(IMG_UINT32 uOpcode)
{
	return g_asUseInstDesc[uOpcode].uDescFlags;
}

/******************************************************************************
 End of file (usetab.c)
******************************************************************************/

